"""
This module initializes the entrypoints library for the Enma application.
It sets up the necessary configurations and imports required for the entrypoints.
"""
from enum import Enum
from typing import Any, Generic, Optional, TypeVar, TypedDict

from enma.application.core.handlers.error import InstanceError, SourceNotAvailable, SourceWasNotDefined
from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.application.core.interfaces.use_case import DTO, IUseCase
from enma.application.use_cases.get_manga import GetMangaRequestDTO, GetMangaResponseDTO, GetMangaUseCase
from enma.application.use_cases.get_random import RandomResponseDTO, RandomUseCase
from enma.application.use_cases.paginate import PaginateRequestDTO, PaginateResponseDTO, PaginateUseCase
from enma.application.use_cases.search_manga import SearchMangaRequestDTO, SearchMangaResponseDTO, SearchMangaUseCase
from enma.domain.entities.manga import Manga
from enma.domain.entities.pagination import Pagination
from enma.domain.entities.search_result import SearchResult
from enma.infra.adapters.repositories.manganato import Manganato
from enma.infra.adapters.repositories.nhentai import NHentai, CloudFlareConfig
from enma.infra.core.interfaces.lib import IEnma

class SourcesEnum(str, Enum):
    ...

class DefaultAvailableSources(SourcesEnum):
    NHENTAI = 'nhentai'
    MANGANATO = 'manganato'

class ExtraConfigs(TypedDict):
    cloudflare_config: CloudFlareConfig

AvailableSources = TypeVar('AvailableSources', bound=SourcesEnum)

class SourceManager(Generic[AvailableSources]):
    def __init__(self, **kwargs) -> None:
        self.__SOURCES: dict[str, IMangaRepository] = {'nhentai': NHentai(config=kwargs.get('cloudflare_config')),
                                                       'manganato': Manganato()}
        self.source = None
        self.source_name = ''
    
    def get_source(self,
                   source_name: AvailableSources | str) -> IMangaRepository:
        
        source_name = source_name.value if isinstance(source_name, Enum) else source_name
        source = self.__SOURCES.get(source_name)

        if source is None:
            raise SourceNotAvailable(f'{source_name} is not an available source.\nAvailable Sources {self.__SOURCES.keys()}')
        
        return source
    
    def set_source(self,
                   source_name: AvailableSources | str) -> None:
        source = self.get_source(source_name=source_name)
        self.source = source
        self.source_name = source_name

    def add_source(self,
                   source_name: str,
                   source: IMangaRepository) -> None:

        if not isinstance(source, IMangaRepository):
            raise InstanceError('Provided source is not an instance of IMangaRepository.')
        
        self.__SOURCES[source_name] = source

def instantiate_source(callable):
    def wrapper(self, *args, **kwargs):
        if self.source_manager.source is not None and \
            self._Enma__current_source_name != self.source_manager.source_name:
            self._Enma__initialize_use_case(source=self.source_manager.source)
            self._Enma__current_source_name = self.source_manager.source_name
        
        return callable(self, *args, **kwargs)
    return wrapper

class Enma(IEnma, Generic[AvailableSources]):
    def __init__(self, 
                 source: Optional[AvailableSources] = None, 
                 **kwargs) -> None:

        self.__get_manga_use_case: Optional[IUseCase[GetMangaRequestDTO, GetMangaResponseDTO]] = None
        self.__search_manga_use_case: Optional[IUseCase[SearchMangaRequestDTO, SearchMangaResponseDTO]] = None
        self.__paginate_use_case: Optional[IUseCase[PaginateRequestDTO, PaginateResponseDTO]] = None
        self.__random_use_case: Optional[IUseCase[Any, RandomResponseDTO]] = None
        self.__current_source_name = None

        self.source_manager = SourceManager[AvailableSources](**kwargs)
        if source is not None: self.__initialize_use_case(source=self.source_manager.get_source(source_name=source))

    def __initialize_use_case(self, source: IMangaRepository) -> None:
        self.__get_manga_use_case = GetMangaUseCase(manga_repository=source)
        self.__search_manga_use_case = SearchMangaUseCase(manga_repository=source)     
        self.__paginate_use_case = PaginateUseCase(manga_repository=source)     
        self.__random_use_case = RandomUseCase(manga_repository=source)     
        
    @instantiate_source
    def get(self, identifier: str) -> Manga | None:
        if self.__get_manga_use_case is None:
            raise SourceWasNotDefined('You must define a source before of performing actions.')

        response = self.__get_manga_use_case.execute(dto=DTO(data=GetMangaRequestDTO(identifier=identifier)))
        
        if not response.found: return
        return response.manga
    
    @instantiate_source
    def search(self, query: str, page: int, **kwargs) -> SearchResult:
        if self.__search_manga_use_case is None:
            raise SourceWasNotDefined('You must define a source before of performing actions.')
        
        response = self.__search_manga_use_case.execute(dto=DTO(data=SearchMangaRequestDTO(query=query,
                                                                                           extra=kwargs)))
        
        return response.result
    
    @instantiate_source
    def paginate(self, page: int) -> Pagination:
        if self.__paginate_use_case is None:
            raise SourceWasNotDefined('You must define a source before of performing actions.')
        
        response = self.__paginate_use_case.execute(dto=DTO(data=PaginateRequestDTO(page=page)))

        return response.result
    
    @instantiate_source
    def random(self) -> Manga:
        response = self.__random_use_case.execute() # type: ignore
        return response.result


from dataclasses import dataclass, field
from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.application.core.interfaces.use_case import DTO, IUseCase
from enma.domain.entities.search_result import SearchResult

@dataclass
class SearchMangaRequestDTO:
    query: str
    page: int = field(default=1)
    extra: dict[str, str | int] = field(default_factory=dict)

@dataclass
class SearchMangaResponseDTO:
    result: SearchResult
    
class SearchMangaUseCase(IUseCase[SearchMangaRequestDTO, SearchMangaResponseDTO]):

    def __init__(self, manga_repository: IMangaRepository):
        self.__manga_repository = manga_repository

    def execute(self, dto: DTO[SearchMangaRequestDTO]) -> SearchMangaResponseDTO:
        result = self.__manga_repository.search(query=dto.data.query,
                                                page=dto.data.page,
                                                **dto.data.extra)
        
        return SearchMangaResponseDTO(result=result)


from abc import ABC, abstractmethod

from enma.domain.entities.manga import Manga
from enma.domain.entities.pagination import Pagination
from enma.domain.entities.search_result import SearchResult

class IMangaRepository(ABC):
    @abstractmethod
    def get(self,
            identifier: str) -> Manga | None:
        ...
    
    @abstractmethod
    def search(self,
               query: str,
               page: int,
               **kwargs) -> SearchResult:
        ...

    @abstractmethod
    def paginate(self,
                 page: int) -> Pagination: ...
    
    @abstractmethod
    def random(self) -> Manga:
        ...


"""
This module provides an adapter for the nhentai repository.
It contains functions and classes to interact with the nhentai API and retrieve manga data.
"""

from concurrent.futures import ThreadPoolExecutor
from typing import Any, Optional, cast
from urllib.parse import urlparse, urljoin
from bs4 import BeautifulSoup, Tag

import requests

from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.domain.entities.manga import Chapter, Image, Manga, Title
from enma.domain.entities.search_result import Pagination, SearchResult, Thumb

class Manganato(IMangaRepository):
    """
    Repository class for interacting with the Manganato website.
    Provides methods to fetch manga details, search for manga, etc.
    """
    def __init__(self) -> None:
        self.__BASE_URL = 'https://manganato.com'
        self.__CHAPTER_BASE_URL = 'https://chapmanganato.com'

    def __make_request(self, 
                       url: str,
                       headers: dict[str, Any] | None = None,
                       params: Optional[dict[str, str | int]] = None):

        headers = headers if headers is not None else {}
        params = params if params is not None else {}

        return requests.get(url=urlparse(url).geturl(), 
                            headers={**headers, 'Referer': 'https://chapmanganato.com/'},
                            params={**params})
    
    def __create_title(self, 
                       main_title: str, 
                       alternative: str) -> Title:
        jp, cn, *_ = alternative.split(';') if alternative.find(';') != -1 else alternative.split(',')
        return Title(english=main_title.strip(),
                     japanese=jp.strip(),
                     other=cn.strip())
    
    def __find_chapets_list(self, html: BeautifulSoup) -> list[str]:
        chapter_list = cast(Tag, html.find('ul', {'class': 'row-content-chapter'}))
        chapters = chapter_list.find_all('li') if chapter_list else []
        return [chapter.find('a')['href'] for chapter in chapters]
    
    def __create_chapter(self, url: str) -> Chapter | None:
        response = self.__make_request(url=url)
        
        if response.status_code != 200:
            return
        
        chapter = Chapter(id=response.url.split('/')[-1])
        html = BeautifulSoup(response.text, 'html.parser')
        images_container = cast(Tag, html.find('div'))
        chapter.pages = [Image(uri=img['src'],
                               width=0,
                               height=0) for img in images_container.find_all('img')]
        return chapter

    def get(self, identifier: str) -> Manga | None:
        response = self.__make_request(url=urljoin(self.__CHAPTER_BASE_URL, identifier))
        
        if response.status_code != 200:
            return
        
        soup = BeautifulSoup(response.text, 'html.parser')

        title: Title = Title()
        author: Optional[str] = None
        genres: list[str] = []
        cover = None

        elem_title = ''

        cover_img = cast(Tag, soup.find('span', {'class': 'info-image'}))
        if cover_img: 
            cover_tag = cast(Tag, cover_img.find('img'))
            if cover_tag:
                cover = cover_tag['src']

        title_div = soup.find('div', {'class': 'story-info-right'})
        if title_div is not None:
            h1_elem = title_div.find('h1')
            h1_tag = cast(Tag, h1_elem)
            h1_text = h1_tag.text if h1_elem is not None else ''
            elem_title = h1_text
        
        table = cast(Tag, soup.find('table', {'class': 'variations-tableInfo'}))
        table_vals = table.find_all('td', {'class': 'table-value'})
        
        if table_vals:
            title_cel, author_cel, _, genres_cel = table_vals
            title = self.__create_title(main_title=elem_title, 
                                        alternative=title_cel.text)
            author = author_cel.text.strip()
            genres = genres_cel.text.replace('\n', '').split(' - ')

        with ThreadPoolExecutor(max_workers=10) as executor:
            chapters = executor.map(self.__create_chapter, self.__find_chapets_list(html=soup))
            chapters = list(filter(lambda x: isinstance(x, Chapter), list(chapters)))
        
        return Manga(title=title,
                     author=author,
                     genres=genres,
                     id=identifier,
                     thumbnail=None,
                     cover=cover, # type: ignore
                     chapters=cast(list[Chapter], chapters))
    
    def search(self, 
               query: str,
               page: int) -> SearchResult:
        
        response = self.__make_request(url=urljoin(f'{self.__BASE_URL}/search/story/', "_".join(query.split())),
                                       params={'page': page})
        
        if response.status_code != 200:
            return SearchResult(query=query,
                                page=page)
        
        soup = BeautifulSoup(response.text, 'html.parser')

        total_pages = 0
        total_pages_elem = soup.find('a', {'class': 'page-blue page-last'})
        if total_pages_elem:
            total_pages = int(total_pages_elem['href'].split('page=')[-1]) # type: ignore

        results = soup.find_all('div', {'class': 'search-story-item'})

        thumbs: list[Thumb] = []

        for result in results:
            thumbs.append(Thumb(title=result.find('h3').text.replace('\n', '').strip(),
                                cover=Image(uri=result.find('img')['src'], width=0, height=0),
                                id=result.find('a', {'class': 'a-h text-nowrap item-title'})['href'].split('/')[-1]))

        return SearchResult(query=query,
                            page=page,
                            total_pages=total_pages,
                            results=thumbs)

    def paginate(self, page: int) -> Pagination:
        return super().paginate(page)
    
    def random(self) -> Manga:
        return super().random()



from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Optional, TypedDict
from enma.domain.entities.base import Entity

@dataclass
class Image:
    uri: str
    width: int
    height: int

class MIME(Enum):
    J = 'jpg'
    P = 'png'
    G = 'gif'

@dataclass
class Title:
    english: str = field(default='')
    japanese: str = field(default='')
    other: str = field(default='')

class IMangaProps(TypedDict):
    id: int | str
    created_at: datetime
    updated_at: datetime
    title: Title
    pages_count: int
    pages: list[Image]

@dataclass
class Chapter:
    id: str | int
    pages: list[Image] = field(default_factory=list)

    def add_page(self, page: Image) -> None:
        self.pages.append(page)

@dataclass
class Manga(Entity[IMangaProps]):

    title: Title
    author: str | None
    genres: list[str]
    chapters: list[Chapter]
    chapters_count: int
    cover: Image | None
    thumbnail: Image | None

    def __init__(self,
                 title: Title,
                 chapters: list[Chapter],
                 genres: list[str] | None = None,
                 author: str | None = None,
                 thumbnail: Image | None = None,
                 cover: Image | None = None,
                 id: int | str | None = None, 
                 created_at: datetime | None = None, 
                 updated_at: datetime | None = None):
        
        super().__init__(id=id,
                         created_at=created_at,
                         updated_at=updated_at)
        
        self.title = title
        self.chapters = chapters
        self.thumbnail = thumbnail
        self.cover = cover
        self.chapters_count = len(self.chapters if self.chapters else [])
        self.author = author
        self.genres = genres or []
        


from dataclasses import asdict, dataclass
from datetime import datetime
import json
from typing import Generic, TypeVar, Union

T = TypeVar('T')

@dataclass
class Entity(Generic[T]):
    """Base class for entities in the domain model.

    This class provides common attributes and methods for all entities.

    Attributes:
        id: A Union of int and str representing the entity's ID.
        created_at: A datetime object representings when the entity was created.
        updated_at: A datetime object representing when the entity was last updated.
    """

    id: Union[int, str]
    created_at: datetime
    updated_at: datetime

    def __init__(self, 
                 id: Union[int, str, None] = None,
                 created_at: Union[datetime, None] = None,
                 updated_at: Union[datetime, None] = None) -> None:
        """Initializes an Entity with given or default values.

        Args:
            id: A Union of int, str, and None representing the entity's ID. Defaults to 0.
            created_at: A Union of datetime and None representing when the entity was created. Defaults to current UTC time.
            updated_at: A Union of datetime and None representing when the entity was last updated. Defaults to current UTC time.
        """

        self.id = id if id is not None else 0
        self.created_at = created_at if created_at is not None else datetime.utcnow()
        self.updated_at = updated_at if updated_at is not None else datetime.utcnow()

    def to_dict(self) -> T:
        """Converts the entity to a dictionary.

        Returns:
            A dictionary representation of the entity.
        """
        return asdict(self) # type: ignore
    
    def to_json(self) -> T:
        """Converts the entity to a json.

        Returns:
            A json representation of the entity.
        """
        return json.dumps(self.to_dict(),
                          ensure_ascii=False,
                          default=lambda x: x.isoformat()) # type: ignore


"""
This module provides an adapter for the nhentai repository.
It contains functions and classes to interact with the nhentai API and retrieve manga data.
"""

from dataclasses import dataclass
from typing import Any, Literal, Optional, cast
from urllib.parse import urlparse, urljoin
from enum import Enum
from bs4 import BeautifulSoup, Tag

import requests

from enma.application.core.handlers.error import ExceedRetryCount, NhentaiSourceWithoutConfig
from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.domain.entities.manga import Image, Manga, MIME, Title, Chapter
from enma.domain.entities.search_result import Pagination, SearchResult, Thumb

@dataclass
class CloudFlareConfig:
    user_agent: str
    cf_clearance: str

class __StrEnum(str, Enum):...

class Sort(__StrEnum):
    TODAY = 'popular-today'
    WEEK = 'popular-week'
    ALL_TIME = 'popular'
    RECENT = None

class NHentai(IMangaRepository):
    """
    Repository class for interacting with the nhentai API.
    Provides methods to fetch manga details, search for manga, etc.
    """
    def __init__(self,
                 config: Optional[CloudFlareConfig] = None) -> None:
        self.__config = config
        self.__BASE_URL = 'https://nhentai.net/'
        self.__API_URL = 'https://nhentai.net/api/'
        self.__IMAGE_BASE_URL = 'https://i.nhentai.net/galleries/'
        self.__AVATAR_URL = 'https://i5.nhentai.net/'
        self.__TINY_IMAGE_BASE_URL = self.__IMAGE_BASE_URL.replace('/i.', '/t.')

    def __make_request(self, 
                       url: str,
                       headers: dict[str, Any] | None = None,
                       params: Optional[dict[str, str | int]] = None):

        if self.__config is None:
            raise NhentaiSourceWithoutConfig('Please provide a valid cloudflare cookie and user-agent.')

        headers = headers if headers is not None else {}
        params = params if params is not None else {}

        return requests.get(url=urlparse(url).geturl(), 
                            headers={**headers, 'User-Agent': self.__config.user_agent},
                            params={**params},
                            cookies={'cf_clearance': self.__config.cf_clearance})
    
    def set_config(self, config: CloudFlareConfig) -> None:
        self.__config = config
    
    def __make_page_uri(self, 
                        type: Literal['cover'] | Literal['page'] | Literal['thumbnail'],
                        media_id: str,
                        mime: MIME,
                        page_number: Optional[int] = None) -> str:
        if type == 'cover': return urljoin(self.__TINY_IMAGE_BASE_URL, f'{media_id}/cover.{mime.value}')
        if type == 'thumbnail': return urljoin(self.__TINY_IMAGE_BASE_URL, f'{media_id}/thumb.{mime.value}')
        return urljoin(self.__IMAGE_BASE_URL, f'{media_id}/{page_number}.{mime.value}')

    def get(self, identifier: str) -> Manga | None:
        response = self.__make_request(url=f'{self.__API_URL}/gallery/{identifier}')
        
        if response.status_code != 200:
            return
        
        doujin = response.json()

        chapter = Chapter(id=0)
        
        for index, page in enumerate(doujin.get('images').get('pages')):
            page = Image(uri=self.__make_page_uri(type='page',
                                                  mime=MIME[doujin.get("images").get("thumbnail").get("t").upper()],
                                                  media_id=doujin.get('media_id'),
                                                  page_number=index+1),
                         width=page.get('w'),
                         height=page.get('h'))
            
            chapter.add_page(page=page)
        
        manga = Manga(title=Title(english=doujin.get('title').get('english'),
                                  japanese=doujin.get('title').get('japanese'),
                                  other=doujin.get('title').get('pretty')),
                      id=doujin.get('id'),
                      thumbnail=Image(uri=self.__make_page_uri(type='thumbnail',
                                                               mime=MIME[doujin.get("images").get("thumbnail").get("t").upper()],
                                                               media_id=doujin.get('media_id')),
                                      width=doujin.get("images").get("thumbnail").get("w"),
                                      height=doujin.get("images").get("thumbnail").get("h")),
                      cover=Image(uri=self.__make_page_uri(type='cover',
                                                            media_id=doujin.get('media_id'),
                                                            mime=MIME[doujin.get("images").get("thumbnail").get("t").upper()]),
                                  width=doujin.get("images").get("cover").get("w"),
                                  height=doujin.get("images").get("cover").get("h")),
                      chapters=[chapter])

        return manga
    
    def search(self,
               query: str, 
               page: int, 
               sort: Sort = Sort.RECENT) -> SearchResult:
        
        request_response = self.__make_request(url=urljoin(self.__BASE_URL, 'search'),
                                               params={'q': query,
                                                       'sort': sort if isinstance(sort, str) else sort.value,
                                                       'page': page})
        
        soup = BeautifulSoup(request_response.text, 'html.parser')

        search_results_container = soup.find('div', {'class': 'container'})
        pagination_container = soup.find('section', {'class': 'pagination'})
    
        last_page_a_tag = pagination_container.find('a', {'class': 'last'}) if pagination_container else None # type: ignore
        total_pages = int(last_page_a_tag['href'].split('=')[-1]) if last_page_a_tag else 1 # type: ignore
        
        if not search_results_container:
            return SearchResult(query=query,
                                total_pages=total_pages,
                                page=page,
                                total_results=0,
                                results=[])
        
        search_results = search_results_container.find_all('div', {'class': 'gallery'}) # type: ignore

        if not search_results:
            return SearchResult(query=query,
                                total_pages=total_pages,
                                page=page,
                                total_results=0,
                                results=[])
        
        a_tags_with_doujin_id = [gallery.find('a', {'class': 'cover'}) for gallery in search_results]

        thumbs = []

        for a_tag in a_tags_with_doujin_id:
            if a_tag is None: continue

            doujin_id = a_tag['href'].split('/')[-2]
            
            if doujin_id == '': continue
            
            result_cover = a_tag.find('img', {'class': 'lazyload'})
            cover_uri = None
            width = None
            height = None

            if result_cover is not None:
                cover_uri = result_cover['data-src']
                width = result_cover['width']
                height = result_cover['height']

            result_caption = a_tag.find('div', {'class': 'caption'})

            caption = None
            if result_caption is not None:
                caption = result_caption.text
            
            thumbs.append(Thumb(id=doujin_id,
                                cover=Image(uri=cover_uri or '',
                                            width=width or 0,
                                            height=height or 0),
                                title=caption or ''))
        
        return SearchResult(query=query,
                            total_pages=total_pages,
                            page=page,
                            total_results=25*total_pages if pagination_container else len(thumbs),
                            results=thumbs)
    
    def paginate(self, page: int) -> Pagination:
        response = self.__make_request(url=urljoin(self.__API_URL, f'galleries/all'),
                                       params={'page': page})
        
        if response.status_code != 200:
            return Pagination(page=page)
        
        data = response.json()

        PAGES = data.get('num_pages', 0)
        PER_PAGE = data.get('per_page', 0)
        TOTAL_RESULTS = int(PAGES) * int(PER_PAGE)

        return Pagination(page=int(page),
                          total_results=TOTAL_RESULTS,
                          total_pages=PAGES,
                          results=[Thumb(id=result.get('id'),
                                         title=result.get('title').get('english'),
                                         cover=Image(uri=self.__make_page_uri(type='cover', 
                                                                              media_id=result.get('media_id'), 
                                                                              mime=MIME[result.get('images').get('cover').get('t').upper()]),
                                                     width=result.get('images').get('cover').get('w'),
                                                     height=result.get('images').get('cover').get('h'))) for result in data.get('result')])
    
    def random(self, retry=0) -> Manga:
        response = self.__make_request(url=urljoin(self.__BASE_URL, 'random'))
        
        soup = BeautifulSoup(response.text, 'html.parser')

        id = cast(Tag, soup.find('h3', id='gallery_id')).text.replace('#', '')

        doujin = self.get(identifier=id)

        if doujin is None:
            if retry == 4:
                raise ExceedRetryCount('Could not fetch a random doujin.')
            return self.random(retry=retry+1)

        return doujin


from dataclasses import dataclass
from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.application.core.interfaces.use_case import DTO, IUseCase
from enma.domain.entities.pagination import Pagination

@dataclass
class PaginateRequestDTO:
    page: int

@dataclass
class PaginateResponseDTO:
    result: Pagination
    
class PaginateUseCase(IUseCase[PaginateRequestDTO, PaginateResponseDTO]):

    def __init__(self, manga_repository: IMangaRepository):
        self.__manga_repository = manga_repository

    def execute(self, dto: DTO[PaginateRequestDTO]) -> PaginateResponseDTO:
        result = self.__manga_repository.paginate(page=dto.data.page)
        
        return PaginateResponseDTO(result=result)


"""
This module defines the core interfaces for the Enma application's infrastructure layer.
It provides abstract classes and methods that need to be implemented by the concrete classes.
"""
from abc import ABC, abstractmethod

from enma.domain.entities.manga import Manga
from enma.domain.entities.pagination import Pagination
from enma.domain.entities.search_result import SearchResult

class IEnma(ABC):
    @abstractmethod
    def get(self, identifier: str) -> Manga:
        ...
    
    @abstractmethod
    def search(self, query: str, page: int, **kwargs) -> SearchResult:
        ...
    
    @abstractmethod
    def paginate(self, page: int) -> Pagination:
        ...

    @abstractmethod
    def random(self) -> Manga:
        ...


"""
This module defines the SearchResult entity for the Enma application.
It represents the result of a manga search operation.
"""
from dataclasses import dataclass
from datetime import datetime
from typing import TypedDict
from enma.domain.entities.base import Entity
from enma.domain.entities.manga import Image, Manga

@dataclass
class Thumb:
    id: str
    title: str
    cover: Image

class ISearchResultProps(TypedDict):
    id: int | str
    created_at: datetime
    updated_at: datetime
    page: int
    total_pages: int
    total_results: int
    results: list[Manga]

@dataclass
class Pagination(Entity[ISearchResultProps]):
    """
    Entity class representing a search result in the Enma application.
    """
    page: int
    total_pages: int
    total_results: int
    results: list[Thumb]

    def __init__(self,
                 page: int,
                 total_pages: int = 0,
                 total_results: int = 0,
                 results: list[Thumb] | None = None,
                 id: int | str | None = None, 
                 created_at: datetime | None = None, 
                 updated_at: datetime | None = None):
        
        super().__init__(id=id,
                         created_at=created_at,
                         updated_at=updated_at)
        
        self.page = page
        self.results = results or list()
        self.total_pages = total_pages or 1 if len(self.results) <= 20 else total_pages
        self.total_results = total_results or 20 * self.total_pages if self.total_pages > 1 else len(self.results) 
        


import sys
from enma.infra.entrypoints.lib import Enma, SourcesEnum
from enma.infra.adapters.repositories.nhentai import CloudFlareConfig, NHentai, Sort
from enma.infra.adapters.repositories.manganato import Manganato

__version__ = '2.0.0'

package_name = "enma"
python_major = "3"
python_minor = "9"

try:
    assert sys.version_info >= (int(python_major), int(python_minor))
except AssertionError:
    raise RuntimeError(f"\033[31m{package_name!r} requires Python {python_major}.{python_minor}+ (You have Python {sys.version})\033[0m")



from setuptools import setup, find_packages

def readme():
    with open('./README.md', 'r') as readme:
        return readme.read()

def make_setup():
    return setup(
        name='Enma',
        version="2.0.1",
        description='Enma is a Python library designed to fetch manga and doujinshi data from various sources.',
        long_description=readme(),
        long_description_content_type='text/markdown',
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        url='https://github.com/AlexandreSenpai/Enma',
        author='AlexandreSenpai',
        author_email='alexandrebsramos@hotmail.com',
        keywords='Tags, hentai, nhentai, nhentai.net, API, NSFW, erohoshi, Manganato, Doujinshi, Manga, Scrapper',
        license='MIT',
        packages=find_packages(),
        install_requires=['requests', 'beautifulsoup4'],
        include_package_data=True,
        zip_safe=False
    )

if __name__ == '__main__':
    make_setup()


from dataclasses import dataclass
from typing import Union
from enma.application.core.interfaces.manga_repository import IMangaRepository
from enma.application.core.interfaces.use_case import DTO, IUseCase
from enma.domain.entities.manga import Manga

@dataclass
class GetMangaRequestDTO:
    identifier: str

@dataclass
class GetMangaResponseDTO:
    found: bool
    manga: Union[Manga, None]
    
class GetMangaUseCase(IUseCase[GetMangaRequestDTO, GetMangaResponseDTO]):

    def __init__(self, manga_repository: IMangaRepository):
        self.__manga_repository = manga_repository

    def execute(self, dto: DTO[GetMangaRequestDTO]) -> GetMangaResponseDTO:
        manga = self.__manga_repository.get(identifier=dto.data.identifier)
        
        if manga is None: return GetMangaResponseDTO(found=False, manga=None)
        
        return GetMangaResponseDTO(found=True, manga=manga)